# Generated by Django 3.1.2 on 2020-10-26 22:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("eveuniverse", "0003_evemarketprice"),
    ]

    operations = [
        migrations.AlterField(
            model_name="evedogmaeffect",
            name="name",
            field=models.CharField(
                db_index=True, default="", help_text="Eve Online name", max_length=400
            ),
        ),
    ]
