SET FOREIGN_KEY_CHECKS=0;
DROP TABLE IF EXISTS eveuniverse_eveancestry;
DROP TABLE IF EXISTS eveuniverse_eveasteroidbelt ;
DROP TABLE IF EXISTS eveuniverse_evebloodline ;
DROP TABLE IF EXISTS eveuniverse_evecategory ;
DROP TABLE IF EXISTS eveuniverse_eveconstellation ;
DROP TABLE IF EXISTS eveuniverse_evedogmaattribute ;
DROP TABLE IF EXISTS eveuniverse_evedogmaeffect ;
DROP TABLE IF EXISTS eveuniverse_evedogmaeffectmodifier ;
DROP TABLE IF EXISTS eveuniverse_eveentity;
DROP TABLE IF EXISTS eveuniverse_evefaction ;
DROP TABLE IF EXISTS eveuniverse_evegroup ;
DROP TABLE IF EXISTS eveuniverse_evegraphic ;
DROP TABLE IF EXISTS eveuniverse_evemarketgroup;
DROP TABLE IF EXISTS eveuniverse_evemoon ;
DROP TABLE IF EXISTS eveuniverse_eveplanet ;
DROP TABLE IF EXISTS eveuniverse_everace ;
DROP TABLE IF EXISTS eveuniverse_everegion ;
DROP TABLE IF EXISTS eveuniverse_evesolarsystem ;
DROP TABLE IF EXISTS eveuniverse_evestar ;
DROP TABLE IF EXISTS eveuniverse_evestargate;
DROP TABLE IF EXISTS eveuniverse_evestation;
DROP TABLE IF EXISTS eveuniverse_evestationservice;
DROP TABLE IF EXISTS eveuniverse_evestation_services;
DROP TABLE IF EXISTS eveuniverse_evetype;
DROP TABLE IF EXISTS eveuniverse_evetypedogmaattribute;
DROP TABLE IF EXISTS eveuniverse_evetypedogmaeffect;
DROP TABLE IF EXISTS eveuniverse_eveunit;
SET FOREIGN_KEY_CHECKS=1;
